#!/usr/bin/env python

### IPFW Status Monitor
### by Michael 'Curby' Lee
### v1.00, 2008-05-18

###   Usage
#
#  This script is intended to be used with curbywall, but can be used
#  wherever python and ipfw are used.  The full curbywall-ipfw package
#  is available at http://curby.net/filelib/curbywall
#

###   License
#
#  Copyright (C) 2008  Michael Lee <kirbysdl@hotmail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import re, os, sys, time, curses

def main(win):
  # Initialize regular expressions
  re_local     = re.compile('[0-9 ]*count .* // Local (.*)')
  re_main      = re.compile('[0-9 ]*count .* // (.*)')
  re_tokenizer = re.compile('([0-9]+ +[0-9]+ +[0-9]+) +([0-9]+) (.*)')
  re_ports     = re.compile('^[0-9,-]+$')
  re_ips       = re.compile(r'^[0-9/\.,-]+$')

  # Initialize color pairs
  curses.init_pair(1, curses.COLOR_WHITE,   curses.COLOR_BLUE )
  curses.init_pair(2, curses.COLOR_YELLOW,  curses.COLOR_BLACK)
  curses.init_pair(3, curses.COLOR_WHITE,   curses.COLOR_BLACK)
  curses.init_pair(4, curses.COLOR_RED,     curses.COLOR_BLACK)
  curses.init_pair(5, curses.COLOR_GREEN,   curses.COLOR_BLACK)
  curses.init_pair(6, curses.COLOR_CYAN,    curses.COLOR_BLACK)
  curses.init_pair(7, curses.COLOR_MAGENTA, curses.COLOR_BLACK)
  #COLOR_HEADER = curses.A_BOLD + curses.A_UNDERLINE
  COLOR_MHEADER   = curses.color_pair(1) + curses.A_BOLD
  COLOR_LHEADER   = curses.color_pair(1)
  COLOR_STATS     = curses.A_NORMAL
  COLOR_NORM      = curses.color_pair(3)
  COLOR_NO        = curses.color_pair(4) + curses.A_BOLD
  COLOR_MAYBE     = curses.color_pair(2) + curses.A_BOLD
  COLOR_YES       = curses.color_pair(5) + curses.A_BOLD
  COLOR_BODY      = curses.color_pair(6)
  COLOR_PORTS     = curses.color_pair(2)
  COLOR_IPS       = curses.color_pair(2)
  COLOR_DIRECTION = curses.color_pair(7)

  while True:
    # Print a time stamp
    win.addstr("Time: "+time.strftime("%H:%M:%S %F")+"\n\n", COLOR_PORTS)

    # Grab the most recent data
    (ignore, output) = os.popen2(("ipfw", "-T", "show", "0-65534"))

    for line in output:
      # Reformat local rule group headings
      if re_local.match(line):
        line = re_local.sub(r'Local \1                                                                         ', line)
        win.addstr(line[0:80]+"\n", COLOR_LHEADER)

      # Reformat main rule group headings
      elif re_main.match(line):
        line = re_main.sub(r'\1                                                                                ', line)
        win.addstr(line[0:80]+"\n", COLOR_MHEADER)

      # Reformat rules listings
      else:
        lineattr = curses.A_NORMAL
        # Remove uninformative text
        line = line.replace(" from any to any", "")
        line = line.replace(" from any to", " to")
        toks = re_tokenizer.match(line)
        # Mark recently matched rules
        if eval(toks.group(2)) > time.time()-2:
          lineattr += curses.A_REVERSE
        # Pretty-print the rule number and counters
        win.addstr(toks.group(1), lineattr+COLOR_STATS)
        # Pretty-print the rule body
        toks = toks.group(3).split()
        for tok in toks:
          if tok == "allow" or tok == "accept" or tok == "check-state":
            win.addstr(" "+tok, lineattr+COLOR_YES)
          elif tok == "pipe" or tok == "queue" or tok == "fwd" or tok == "forward" or tok == "divert":
            win.addstr(" "+tok, lineattr+COLOR_MAYBE)
          elif tok == "deny" or tok == "drop" or tok == "reset" or tok == "unreach" or tok == "reject" or tok == "log":
            win.addstr(" "+tok, lineattr+COLOR_NO)
          elif re_ports.match(tok):
            win.addstr(" "+tok, lineattr+COLOR_PORTS)
          elif re_ips.match(tok):
            win.addstr(" "+tok, lineattr+COLOR_IPS)
          elif tok == "in" or tok == "out":
            win.addstr(" "+tok, lineattr+COLOR_DIRECTION)
          else:
            win.addstr(" "+tok, lineattr+COLOR_BODY)
        win.addstr("\n")

    # Repaint the screen
    win.refresh()
    win.clear()

    # Wait a second for next iteration
    time.sleep(1)

if __name__=="__main__":
  # Make sure we're root
  if os.getuid() != 0:
    print "You need superuser priveleges to use this script. Try `sudo %s`." % (sys.argv[0])
    sys.exit(1)

  # Initialize the screen for curses, then enter main loop
  win = curses.initscr()
  curses.wrapper(main)
