#!/bin/bash
### Curbywall for IPFW - Status Display
### by Michael 'Curby' Lee
### v2-beta5, 2008-05-17

# For a simple one-time view of the firewall status, type:
#   sudo ipfw -cde show

# This allows multiple people to run versions of this script
# We could also just have one writer and poll from the same file
statusfile="/usr/local/curbywall/status-$$"

if [[ "$UID" != "0" ]]; then
  echo "No root privileges detected.  Attempting \`sudo $0\`..."
  sudo $0
  exit
fi

# Delete the file when exiting
trap "echo Caught SIGINT, deleting $statusfile;  rm $statusfile; exit" SIGINT
trap "echo Caught SIGTERM, deleting $statusfile; rm $statusfile; exit" SIGTERM
trap "echo Caught ERROR, deleting $statusfile;   rm $statusfile; exit" ERR

# Print the status until the cows do something
while true; do
  # The use of a temporary file helps decrease the flickering effect
  ipfw -d show |\
    grep -v ') STATE \|^## Dynamic' |\
    sed 's/^[0-9 ]*count .* \/\/\( Local .*\)/-----\1 -------------------------------------------------------------------------------/' |\
    sed 's/^[0-9 ]*count .* \/\/\(.*\)/=====\1 ===============================================================================/' |\
    sed ';s/^\([=-].\{79\}\).*/\1/;s/ from any to any//;s/ from any to/ to/' > $statusfile
  sleep 1
  clear
  cat $statusfile
done
